package com.persagy.apm.report.detail.paragraphvalue.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.paragraphvalue.dao.ReportParagraphValueMapper;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ReportParagraphValueItemVO;
import com.persagy.apm.report.detail.paragraphvalue.service.IReportParagraphValueService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.report.detail.paragraphvalue.model.*;
import com.persagy.apm.report.detail.paragraphvalue.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 报告段落取值(ReportParagraphValue) service层
 *
 * @author lixing
 * @version V1.0 2021-07-14 16:59:57
 */
@Service
public class ReportParagraphValueServiceImpl extends ServiceImpl<ReportParagraphValueMapper, ReportParagraphValue>
        implements IReportParagraphValueService {

    /**
     * 创建报告段落取值
     *
     * @return 报告段落取值主键
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    @Override
    public String createReportParagraphValue(AddReportParagraphValueDTO addReportParagraphValueDTO) {
        ReportParagraphValue reportParagraphValue = ConvertReportParagraphValueTool.INSTANCE.convertAddDto2Entity(addReportParagraphValueDTO);
        // 设置默认值
        setDefaultValue(reportParagraphValue);
        save(reportParagraphValue);
        return reportParagraphValue.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportParagraphValue 报告段落取值实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportParagraphValue reportParagraphValue) {
        reportParagraphValue.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 报告段落取值详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    @Override
    public ReportParagraphValue queryReportParagraphValueDetail(String id) {
        ReportParagraphValue reportParagraphValue = getById(id);
        if (reportParagraphValue == null) {
            throw new IllegalArgumentException("查看ReportParagraphValue详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportParagraphValue;
    }

    /**
     * 更新报告段落取值
     *
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    @Override
    public void updateReportParagraphValue(UpdateReportParagraphValueDTO updateReportParagraphValueDTO) {
        ReportParagraphValue reportParagraphValue = getById(updateReportParagraphValueDTO.getId());
        reportParagraphValue = ConvertReportParagraphValueTool.INSTANCE.convertUpdateDto2Entity(reportParagraphValue, updateReportParagraphValueDTO);
        reportParagraphValue.setModifier(AppContext.getContext().getAccountId());
        updateById(reportParagraphValue);
    }

    /**
     * 校验报告段落取值是否可删除
     *
     * @param id 报告段落取值主键
     * @return 报告段落取值do类
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    public ReportParagraphValue checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportParagraphValue时发生异常，主键为空");
        }

        ReportParagraphValue reportParagraphValue = getById(id);

        if (reportParagraphValue == null) {
            throw new IllegalArgumentException("删除ReportParagraphValue时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportParagraphValue;
    }

    /**
     * 删除报告段落取值
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    @Override
    public void deleteReportParagraphValue(String id) {
        // 校验是否可删除
        ReportParagraphValue reportParagraphValue = checkDeletable(id);

        reportParagraphValue.setValid(ValidEnum.FALSE.getType());
        updateById(reportParagraphValue);
    }

    /**
     * 查询报告段落取值
     *
     * @return List<ReportParagraphValue>
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    @Override
    public List<ReportParagraphValue> queryReportParagraphValueList(QueryReportParagraphValueDTO queryReportParagraphValueDTO) {
        QueryWrapper<ReportParagraphValue> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportParagraphValue.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportParagraphValue.PROP_CREATIONTIME);

        if (queryReportParagraphValueDTO != null) {

            if (StringUtils.isNotEmpty(queryReportParagraphValueDTO.getReportOutlineId())) {
                queryWrapper.eq(ReportParagraphValue.PROP_REPORT_OUTLINE_ID, queryReportParagraphValueDTO.getReportOutlineId());
            }

            if (StringUtils.isNotEmpty(queryReportParagraphValueDTO.getParagraphId())) {
                queryWrapper.eq(ReportParagraphValue.PROP_PARAGRAPH_ID, queryReportParagraphValueDTO.getParagraphId());
            }

            if (queryReportParagraphValueDTO.getHeader() != null) {
                queryWrapper.eq(ReportParagraphValue.PROP_HEADER, queryReportParagraphValueDTO.getHeader());
            }

            if (queryReportParagraphValueDTO.getContent() != null) {
                queryWrapper.eq(ReportParagraphValue.PROP_CONTENT, queryReportParagraphValueDTO.getContent());
            }

            if (queryReportParagraphValueDTO.getSupplement() != null) {
                queryWrapper.eq(ReportParagraphValue.PROP_SUPPLEMENT, queryReportParagraphValueDTO.getSupplement());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告段落取值
     *
     * @return IPage<ReportParagraphValue>
     * @author lixing
     * @version V1.0 2021-07-14 16:59:57
     */
    @Override
    public IPage<ReportParagraphValue> pageQueryReportParagraphValue(PageQueryReportParagraphValueDTO pageQueryReportParagraphValueDTO) {
        QueryWrapper<ReportParagraphValue> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportParagraphValue.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportParagraphValue> pageParam = new Page<>(pageQueryReportParagraphValueDTO.getPage(), pageQueryReportParagraphValueDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportParagraphValueDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportParagraphValue.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportParagraphValueDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportParagraphValueDTO.getReportOutlineId())) {
            queryWrapper.like(ReportParagraphValue.PROP_REPORT_OUTLINE_ID, pageQueryReportParagraphValueDTO.getReportOutlineId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportParagraphValueDTO.getParagraphId())) {
            queryWrapper.like(ReportParagraphValue.PROP_PARAGRAPH_ID, pageQueryReportParagraphValueDTO.getParagraphId());
        }

        if (pageQueryReportParagraphValueDTO.getHeader() != null) {
            queryWrapper.eq(ReportParagraphValue.PROP_HEADER, pageQueryReportParagraphValueDTO.getHeader());
        }

        if (pageQueryReportParagraphValueDTO.getContent() != null) {
            queryWrapper.eq(ReportParagraphValue.PROP_CONTENT, pageQueryReportParagraphValueDTO.getContent());
        }

        if (pageQueryReportParagraphValueDTO.getSupplement() != null) {
            queryWrapper.eq(ReportParagraphValue.PROP_SUPPLEMENT, pageQueryReportParagraphValueDTO.getSupplement());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public ReportParagraphValueItemVO getParagraphValue(String reportOutlineId, String paragraphId) {
        QueryReportParagraphValueDTO queryReportParagraphValueDTO = new QueryReportParagraphValueDTO();
        queryReportParagraphValueDTO.setReportOutlineId(reportOutlineId);
        queryReportParagraphValueDTO.setParagraphId(paragraphId);
        List<ReportParagraphValue> reportParagraphValues = queryReportParagraphValueList(queryReportParagraphValueDTO);
        if (CollectionUtils.isEmpty(reportParagraphValues)) {
            return null;
        } else {
            ReportParagraphValue reportParagraphValue = reportParagraphValues.get(0);
            return ConvertReportParagraphValueTool.INSTANCE.convert2ResponseItemDTO(reportParagraphValue);
        }
    }

    @Override
    public void deleteByReport(String reportOutlineId) {
        LambdaQueryWrapper<ReportParagraphValue> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ReportParagraphValue::getReportOutlineId, reportOutlineId);
        remove(queryWrapper);
    }

    @Override
    public void updateParagraphSupplement(String reportOutlineId, String paragraphId, List<AttrValueVO> supplement) {
        if (StringUtils.isBlank(reportOutlineId) ||
                StringUtils.isBlank(paragraphId) ||
                CollectionUtils.isEmpty(supplement)) {
            return;
        }

        // 查询需要更新的数据
        QueryReportParagraphValueDTO queryReportParagraphValueDTO = new QueryReportParagraphValueDTO();
        queryReportParagraphValueDTO.setReportOutlineId(reportOutlineId);
        queryReportParagraphValueDTO.setParagraphId(paragraphId);
        List<ReportParagraphValue> reportParagraphValues = queryReportParagraphValueList(queryReportParagraphValueDTO);
        if (CollectionUtils.isEmpty(reportParagraphValues)) {
            return;
        }
        ReportParagraphValue reportParagraphValue = reportParagraphValues.get(0);
        reportParagraphValue.setSupplement(supplement);
        reportParagraphValue.setModifier(AppContext.getContext().getAccountId());
        updateById(reportParagraphValue);
    }
}
