package com.persagy.apm.report.indicator.manualfilling.controller;

import com.persagy.apm.report.indicator.manualfilling.service.IManualFillingDataService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.persagy.apm.report.indicator.manualfilling.model.*;
import com.persagy.apm.report.indicator.manualfilling.model.vo.*;
import com.persagy.apm.report.indicator.manualfilling.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 数据录入数据控制层
 *
 * @author lixing
 * @version V1.0 2021-07-27 09:52:19
 */
@Api(tags = "数据录入数据")
@Validated
@RestController
@RequestMapping("manualFillingData")
@ApiIgnore
public class ManualFillingDataController {
    @Autowired
    private IManualFillingDataService manualFillingDataService;

    @ApiOperation(value = "创建数据录入数据")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createManualFillingData(
            @Valid @RequestBody AddManualFillingDataDTO addManualFillingDataDTO) {
        String id = manualFillingDataService.createManualFillingData(addManualFillingDataDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "数据录入数据详情")
    @PostMapping("/get")
    public CommonResult<ManualFillingDataItemVO> queryManualFillingDataDetail(
            @Valid @RequestBody QueryManualFillingDataDetailDTO queryManualFillingDataDetailDTO) {
        ManualFillingData manualFillingData = manualFillingDataService.queryManualFillingDataDetail(queryManualFillingDataDetailDTO.getId());
        ManualFillingDataItemVO manualFillingDataItemVO = ConvertManualFillingDataTool.INSTANCE.convert2ResponseItemDTO(manualFillingData);
        return ResultHelper.single(manualFillingDataItemVO);
    }


    @ApiOperation(value = "更新数据录入数据")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateManualFillingData(
            @Valid @RequestBody UpdateManualFillingDataDTO updateManualFillingDataDTO) {
        manualFillingDataService.updateManualFillingData(updateManualFillingDataDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateManualFillingDataDTO.getId()));
    }


    @ApiOperation(value = "删除数据录入数据")
    @PostMapping("/delete")
    public CommonResult deleteManualFillingData(
            @RequestBody @Valid DeleteManualFillingDataDTO deleteManualFillingDataDTO) {
        manualFillingDataService.deleteManualFillingData(deleteManualFillingDataDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "数据录入数据列表")
    @PostMapping("/query")
    public CommonResult<PageList<ManualFillingDataListItemVO>> queryManualFillingData(
            @Valid @RequestBody QueryManualFillingDataDTO queryManualFillingDataDTO) {
        List<ManualFillingData> manualFillingDataList = manualFillingDataService.queryManualFillingDataList(queryManualFillingDataDTO);
        List<ManualFillingDataListItemVO> resultList = ConvertManualFillingDataTool.INSTANCE.convert2List(manualFillingDataList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "数据录入数据列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ManualFillingDataListItemVO>> queryManualFillingData(
    //        @Valid @RequestBody PageQueryManualFillingDataDTO pageQueryManualFillingDataDTO) {
    //    IPage<ManualFillingData> pagedList = manualFillingDataService.pageQueryManualFillingData(pageQueryManualFillingDataDTO);
    //    IPage<ManualFillingDataListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertManualFillingDataTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，ManualFillingData实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

