package com.persagy.apm.report.indicator.manualfilling.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.report.indicator.manualfilling.dao.ManualFillingDataMapper;
import com.persagy.apm.report.indicator.manualfilling.model.ConvertManualFillingDataTool;
import com.persagy.apm.report.indicator.manualfilling.model.ManualFillingData;
import com.persagy.apm.report.indicator.manualfilling.model.dto.AddManualFillingDataDTO;
import com.persagy.apm.report.indicator.manualfilling.model.dto.PageQueryManualFillingDataDTO;
import com.persagy.apm.report.indicator.manualfilling.model.dto.QueryManualFillingDataDTO;
import com.persagy.apm.report.indicator.manualfilling.model.dto.UpdateManualFillingDataDTO;
import com.persagy.apm.report.indicator.manualfilling.service.IManualFillingDataService;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

/**
 * 数据录入数据(ManualFillingData) service层
 *
 * @author lixing
 * @version V1.0 2021-07-27 09:52:19
 */
@Service
public class ManualFillingDataServiceImpl extends ServiceImpl<ManualFillingDataMapper, ManualFillingData>
        implements IManualFillingDataService {

    /**
     * 创建数据录入数据
     *
     * @return 数据录入数据主键
     * @author lixing
     * @version V1.0 2021-07-27 09:52:19
     */
    @Override
    public String createManualFillingData(AddManualFillingDataDTO addManualFillingDataDTO) {
        ManualFillingData manualFillingData = ConvertManualFillingDataTool.INSTANCE.convertAddDto2Entity(addManualFillingDataDTO);
        // 设置默认值
        setDefaultValue(manualFillingData);
        save(manualFillingData);
        return manualFillingData.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param manualFillingData 数据录入数据实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ManualFillingData manualFillingData) {
        manualFillingData.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 数据录入数据详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-07-27 09:52:19
     */
    @Override
    public ManualFillingData queryManualFillingDataDetail(String id) {
        ManualFillingData manualFillingData = getById(id);
        if (manualFillingData == null) {
            throw new IllegalArgumentException("查看ManualFillingData详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return manualFillingData;
    }

    /**
     * 更新数据录入数据
     *
     * @author lixing
     * @version V1.0 2021-07-27 09:52:19
     */
    @Override
    public void updateManualFillingData(UpdateManualFillingDataDTO updateManualFillingDataDTO) {
        ManualFillingData manualFillingData = getById(updateManualFillingDataDTO.getId());
        manualFillingData = ConvertManualFillingDataTool.INSTANCE.convertUpdateDto2Entity(manualFillingData, updateManualFillingDataDTO);
        manualFillingData.setModifier(AppContext.getContext().getAccountId());
        updateById(manualFillingData);
    }

    /**
     * 校验数据录入数据是否可删除
     *
     * @param id 数据录入数据主键
     * @return 数据录入数据do类
     * @author lixing
     * @version V1.0 2021-07-27 09:52:19
     */
    public ManualFillingData checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ManualFillingData时发生异常，主键为空");
        }

        ManualFillingData manualFillingData = getById(id);

        if (manualFillingData == null) {
            throw new IllegalArgumentException("删除ManualFillingData时发生异常，找不到要删除的数据，id:" + id);
        }

        return manualFillingData;
    }

    /**
     * 删除数据录入数据
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-07-27 09:52:19
     */
    @Override
    public void deleteManualFillingData(String id) {
        // 校验是否可删除
        ManualFillingData manualFillingData = checkDeletable(id);

        manualFillingData.setValid(ValidEnum.FALSE.getType());
        updateById(manualFillingData);
    }

    /**
     * 查询数据录入数据
     *
     * @return List<ManualFillingData>
     * @author lixing
     * @version V1.0 2021-07-27 09:52:19
     */
    @Override
    public List<ManualFillingData> queryManualFillingDataList(QueryManualFillingDataDTO queryManualFillingDataDTO) {
        QueryWrapper<ManualFillingData> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ManualFillingData.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ManualFillingData.PROP_CREATIONTIME);

        if (queryManualFillingDataDTO != null) {
            if (!CollectionUtils.isEmpty(queryManualFillingDataDTO.getFunctionIds())) {
                queryWrapper.in(ManualFillingData.PROP_FUNCTION_ID, queryManualFillingDataDTO.getFunctionIds());
            }

            if (StringUtils.isNotEmpty(queryManualFillingDataDTO.getFunctionId())) {
                queryWrapper.eq(ManualFillingData.PROP_FUNCTION_ID, queryManualFillingDataDTO.getFunctionId());
            }

            if (StringUtils.isNotEmpty(queryManualFillingDataDTO.getProjectId())) {
                queryWrapper.eq(ManualFillingData.PROP_PROJECT_ID, queryManualFillingDataDTO.getProjectId());
            }

            if (queryManualFillingDataDTO.getDataTime() != null) {
                queryWrapper.eq(ManualFillingData.PROP_DATA_TIME, queryManualFillingDataDTO.getDataTime());
            }

            if (queryManualFillingDataDTO.getStartTime() != null) {
                queryWrapper.ge(ManualFillingData.PROP_DATA_TIME, queryManualFillingDataDTO.getStartTime());
            }

            if (queryManualFillingDataDTO.getEndTime() != null) {
                queryWrapper.le(ManualFillingData.PROP_DATA_TIME, queryManualFillingDataDTO.getEndTime());
            }


            if (StringUtils.isNotEmpty(queryManualFillingDataDTO.getType())) {
                queryWrapper.eq(ManualFillingData.PROP_TYPE, queryManualFillingDataDTO.getType());
            }

            if (StringUtils.isNotEmpty(queryManualFillingDataDTO.getValue())) {
                queryWrapper.eq(ManualFillingData.PROP_VALUE, queryManualFillingDataDTO.getValue());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询数据录入数据
     *
     * @return IPage<ManualFillingData>
     * @author lixing
     * @version V1.0 2021-07-27 09:52:19
     */
    @Override
    public IPage<ManualFillingData> pageQueryManualFillingData(PageQueryManualFillingDataDTO pageQueryManualFillingDataDTO) {
        QueryWrapper<ManualFillingData> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ManualFillingData.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ManualFillingData> pageParam = new Page<>(pageQueryManualFillingDataDTO.getPage(), pageQueryManualFillingDataDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryManualFillingDataDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ManualFillingData.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryManualFillingDataDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryManualFillingDataDTO.getFunctionId())) {
            queryWrapper.like(ManualFillingData.PROP_FUNCTION_ID, pageQueryManualFillingDataDTO.getFunctionId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryManualFillingDataDTO.getProjectId())) {
            queryWrapper.like(ManualFillingData.PROP_PROJECT_ID, pageQueryManualFillingDataDTO.getProjectId());
        }

        if (pageQueryManualFillingDataDTO.getDataTime() != null) {
            queryWrapper.eq(ManualFillingData.PROP_DATA_TIME, pageQueryManualFillingDataDTO.getDataTime());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryManualFillingDataDTO.getType())) {
            queryWrapper.like(ManualFillingData.PROP_TYPE, pageQueryManualFillingDataDTO.getType());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryManualFillingDataDTO.getValue())) {
            queryWrapper.like(ManualFillingData.PROP_VALUE, pageQueryManualFillingDataDTO.getValue());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public void batchDelete(String functionId, String projectId, List<Date> deleteDates) {
        if (StringUtils.isBlank(functionId) || StringUtils.isBlank(projectId) || CollectionUtils.isEmpty(deleteDates)) {
            return;
        }
        QueryWrapper<ManualFillingData> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ManualFillingData.PROP_FUNCTION_ID, functionId);
        queryWrapper.eq(ManualFillingData.PROP_PROJECT_ID, projectId);
        queryWrapper.in(ManualFillingData.PROP_DATA_TIME, deleteDates);
        remove(queryWrapper);
    }

    @Override
    public void batchSave(List<ManualFillingData> manualFillingDataList) {
        if (CollectionUtils.isEmpty(manualFillingDataList)) {
            return;
        }
        saveBatch(manualFillingDataList);
    }
}
