package com.persagy.apm.report.outline.service.impl;

import cn.hutool.core.thread.ExecutorBuilder;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.outline.service.builder.IReportGenerator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.concurrent.*;

/**
 * 消费消息线程池
 *
 * @author lixing
 * @version V1.0 2021/5/12 2:53 下午
 **/
@Component
@Slf4j
public class GenerateReportThreadPool {
    @Autowired
    IReportGenerator reportGenerator;
    @Value("${maxGenerateMinutes:5}")
    private int maxGenerateMinutes;

    private final ExecutorService es;

    public GenerateReportThreadPool() {
        // 初始化线程池
        es = ExecutorBuilder.create()
                .setCorePoolSize(5)
                .setMaxPoolSize(10)
                .setWorkQueue(new LinkedBlockingQueue<>(1000))
                .setHandler(new ThreadPoolExecutor.CallerRunsPolicy())
                .build();
    }

    /**
     * 生成报告
     *
     * @param reportOutline 报告outline对象
     * @author lixing
     * @version V1.0 2021/5/24 4:09 下午
     */
    public void generateReport(ReportOutline reportOutline) {
        String userId = PoemsContext.getContext().getUserId();
        String pd = PoemsContext.getContext().getPd();
        String loginDevice = PoemsContext.getContext().getLoginDevice();
        // 添加Future任务，过期时间为5分钟
        FutureTask<String> future = new FutureTask<>(() -> {
            PoemsContext.setContext(userId, loginDevice, pd);
            synchronized (reportOutline.getId().intern()) {
                reportGenerator.generateReport(reportOutline);
            }
            return "done";
        });

        es.execute(future);
        es.execute(() -> {
            try {
                future.get(maxGenerateMinutes, TimeUnit.MINUTES);
            } catch (InterruptedException | ExecutionException | TimeoutException e) {
                log.error("生成报告发生异常", e);
                future.cancel(true);
                reportGenerator.updateReportOutlineInfoWhenFailure(reportOutline);
            }
        });
    }
}
