package com.persagy.apm.report.paragraphs.service.impl;

import com.persagy.apm.report.userconfig.functiongroup.service.IFunctionGroupService;
import com.persagy.apm.report.userconfig.paragraph.model.Paragraph;
import com.persagy.apm.report.userconfig.paragraph.service.IParagraphService;
import com.persagy.apm.report.userconfig.paragraphcustom.service.IParagraphCustomService;
import com.persagy.apm.report.userconfig.pargraphdefine.service.IParagraphDefineService;
import com.persagy.apm.report.userconfig.reporttype.service.IReportTypeService;
import com.persagy.apm.report.dependencies.saasweb.model.vo.PartitionVO;
import com.persagy.apm.report.dependencies.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.report.detail.enums.ParagraphContentItemTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.detail.paragraphvalue.service.IReportParagraphValueService;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.outline.service.IReportOutlineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 区域段落（行为项目/分区）建造者实现类
 *
 * @author lixing
 * @version V1.0 2021/7/20 9:42 上午
 **/
public abstract class AreaDepsParagraphBuilder extends ParagraphBuilder {
    @Autowired
    IParagraphDefineService paragraphDefineService;
    @Autowired
    IReportOutlineService reportOutlineService;
    @Autowired
    IFunctionGroupService functionGroupService;
    @Autowired
    IParagraphCustomService paragraphCustomService;
    @Autowired
    IReportTypeService reportTypeService;
    @Autowired
    IReportParagraphValueService paragraphValueService;
    @Autowired
    IParagraphService paragraphService;

    public abstract ProjectParagraphBuilder getProjectParagraphBuilder();

    /**
     * 分区下是分区，获取分区的段落内容取值信息
     *
     * @param paragraph     段落对象
     * @param reportOutline 报告概要对象
     * @param sonAreas      分区下的子分区列表
     * @return 分区下是分区，获取分区的段落内容取值信息
     * @author lixing
     * @version V1.0 2021/7/22 11:47 上午
     */
    private ParagraphContentVO getSonAreaParagraphContent(
            Paragraph paragraph, ReportOutline reportOutline, List<PartitionVO> sonAreas) {
        List<ItemVO> sonAreaItems = new ArrayList<>();
        List<String> projectIdsInArea = new ArrayList<>();
        for (PartitionVO sonArea : sonAreas) {
            List<AttrValueVO> sonAreaSummary = null;
            List<SimpleProjectVO> simpleProjects = reportOutlineService.queryAreaSonProjects(
                    reportOutline, sonArea.getAreaId());
            if (!CollectionUtils.isEmpty(simpleProjects)) {
                List<String> projectIdsInSonArea = simpleProjects.stream().map(
                        SimpleProjectVO::getProjectId).collect(Collectors.toList());
                List<ItemVO> projectSummaries = simpleProjects.stream().map(simpleProjectVO -> {
                    ItemVO itemVO = new ItemVO();
                    ParagraphContentVO projectParagraphContent = getProjectParagraphBuilder().getParagraphContentVO(
                            paragraph, simpleProjectVO.getProjectId(), reportOutline.getReportMonth());
                    itemVO.setAttrs(projectParagraphContent.getSummary());
                    return itemVO;
                }).collect(Collectors.toList());
                // 计算子分区的汇总信息
                sonAreaSummary = getSummary(null, projectSummaries, projectIdsInSonArea);
                projectIdsInArea.addAll(projectIdsInSonArea);
            }
            ItemVO itemVO = new ItemVO();
            itemVO.setName(sonArea.getAreaName());
            itemVO.setAttrs(sonAreaSummary);
            sonAreaItems.add(itemVO);
        }
        ParagraphContentVO paragraphContent = new ParagraphContentVO();
        paragraphContent.setItems(sonAreaItems);
        paragraphContent.setSummary(getSummary(null, sonAreaItems, projectIdsInArea));
        return paragraphContent;
    }

    /**
     * 分区下是项目，获取分区的段落内容取值信息
     *
     * @param paragraph     段落对象
     * @param reportOutline 报告概要对象
     * @return 分区下是项目，分区的段落内容取值信息
     * @author lixing
     * @version V1.0 2021/7/22 11:47 上午
     */
    public ParagraphContentVO getProjectParagraphContent(Paragraph paragraph, ReportOutline reportOutline) {
        ParagraphContentVO paragraphContent = null;
        // 获取报告相关的项目列表
        List<SimpleProjectVO> simpleProjectList = reportOutlineService.queryRelatedProjects(reportOutline);
        if (simpleProjectList != null) {
            paragraphContent = new ParagraphContentVO();
            List<ItemVO> items = new ArrayList<>();
            for (SimpleProjectVO simpleProjectVO : simpleProjectList) {
                ParagraphContentVO projectParagraphContent = getProjectParagraphBuilder().getParagraphContentVO(
                        paragraph, simpleProjectVO.getProjectId(), reportOutline.getReportMonth());
                items.add(getProjectParagraphBuilder().getProjectValue(
                        simpleProjectVO.getProjectId(), simpleProjectVO.getProjectLocalName(), projectParagraphContent));
            }
            List<String> projectIdsInArea = simpleProjectList.stream().map(
                    SimpleProjectVO::getProjectId).collect(Collectors.toList());
            paragraphContent.setItems(items);
            paragraphContent.setSummary(getSummary(null, items, projectIdsInArea));
        }
        return paragraphContent;
    }

    /**
     * 计算段落内容部分
     *
     * @param paragraph 段落对象
     * @param reportOutline 报告概要实体
     * @return 段落内容部分
     * @author lixing
     * @version V1.0 2021/8/2 9:43 上午
     */
    @Override
    public ParagraphContentVO getParagraphContent(Paragraph paragraph, ReportOutline reportOutline) {
        ParagraphContentVO paragraphContent;
        // 获取分区下的子分区
        List<PartitionVO> sonAreas = reportTypeService.getSonAreas(
                reportOutline.getReportTypeId(), reportOutline.getBelong());
        if (CollectionUtils.isEmpty(sonAreas)) {
            paragraphContent = getProjectParagraphContent(paragraph, reportOutline);
            paragraphContent.setItemType(ParagraphContentItemTypeEnum.PROJECT);
        } else {
            paragraphContent = getSonAreaParagraphContent(paragraph, reportOutline, sonAreas);
            paragraphContent.setItemType(ParagraphContentItemTypeEnum.AREA);
        }
        return paragraphContent;
    }
}
