package com.persagy.apm.report.paragraphs.service.impl.env;

import com.alibaba.excel.util.CollectionUtils;
import com.google.gson.Gson;
import com.persagy.apm.report.common.constants.DataConstants;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.dependencies.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.report.dependencies.saasweb.service.ISaasWebService;
import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.GroupVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.indicator.codes.env.AreaEnvContentCodes;
import com.persagy.apm.report.indicator.codes.env.AreaEnvHeaderCodes;
import com.persagy.apm.report.indicator.codes.env.ProjectEnvContentCodes;
import com.persagy.apm.report.indicator.factory.EnvFactory;
import com.persagy.apm.report.indicator.model.vo.GraphItemVO;
import com.persagy.apm.report.indicator.model.vo.GraphVO;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.paragraphs.constants.ItemAttrConstants;
import com.persagy.apm.report.paragraphs.constants.StandardNames;
import com.persagy.apm.report.paragraphs.service.impl.AreaIndicatorsParagraphBuilder;
import com.persagy.apm.report.paragraphs.utils.ParagraphUtils;
import com.persagy.apm.report.userconfig.function.service.IFunctionService;
import com.persagy.apm.report.userconfig.statisticsitems.enums.QualifiedEnum;
import org.mockito.internal.util.collections.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 环境多项目段落建造者
 *
 * @author lixing
 * @version V1.0 2021/7/26 4:52 下午
 **/
@Service
public class AreaEnvBuilder extends AreaIndicatorsParagraphBuilder {
    @Autowired
    ProjectEnvBuilder projectEnvBuilder;
    @Autowired
    ISaasWebService saasWebService;
    @Autowired
    EnvFactory envFactory;
    @Autowired
    IFunctionService functionService;

    @Override
    public List<AttrValueVO> getHeader(
            ParagraphContentVO paragraphContentVO, ReportOutline reportOutline) {
        List<AttrValueVO> results = new ArrayList<>();

        // 项目达标率最差的指标
        String worstIndicator = null;
        // 达标率最差指标不合格项目数
        String worstIndicatorNoncomplianceProjectCount = null;
        // 项目达标率最好的指标
        String bestIndicator = null;
        // 达标率最好指标合格项目数
        String bestIndicatorComplianceProjectCount = null;
        // 总项目数
        String projectCount = null;

        List<ItemVO> items = paragraphContentVO.getItems();
        Set<String> attrCodeSet = Sets.newSet(
                AreaEnvContentCodes.projectsCount.name(),
                AreaEnvContentCodes.unqualifiedCount.name(),
                AreaEnvContentCodes.qualifiedCount.name()
        );
        if (!CollectionUtils.isEmpty(items)) {
            // 将段落中包含的条目转换为map，进行排序
            List<Map<String, String>> itemMapList = items.stream().map(
                    item -> ParagraphUtils.item2Map(item, attrCodeSet)).collect(Collectors.toList());

            // 总项目数
            projectCount = itemMapList.get(0).get(AreaEnvContentCodes.projectsCount.name());

            // 项目达标率最差的指标
            List<Map<String, Object>> worstMap = itemMapList.stream().filter(
                    map -> map.get(AreaEnvContentCodes.unqualifiedCount.name()) != null).
                    map(map -> ParagraphUtils.convertMapMemberType(
                            map, AreaEnvContentCodes.unqualifiedCount.name(), Integer.class)
                    ).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(worstMap)) {
                Map<String, Integer> worstSortMap = new HashMap<>();
                worstSortMap.put(AreaEnvContentCodes.unqualifiedCount.name(), -1);
                DataUtils.sort(worstMap, worstSortMap);
                worstIndicator = (String) worstMap.get(0).get(ItemAttrConstants.NAME);
                worstIndicatorNoncomplianceProjectCount = String.valueOf(
                        worstMap.get(0).get(AreaEnvContentCodes.unqualifiedCount.name()));
            }

            // 项目达标率最好的指标
            List<Map<String, Object>> bestMap = itemMapList.stream().filter(
                    map -> map.get(AreaEnvContentCodes.qualifiedCount.name()) != null).
                    map(map -> ParagraphUtils.convertMapMemberType(
                            map, AreaEnvContentCodes.qualifiedCount.name(), Integer.class)
                    ).collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(bestMap)) {
                Map<String, Integer> bestSortMap = new HashMap<>();
                bestSortMap.put(AreaEnvContentCodes.qualifiedCount.name(), -1);
                DataUtils.sort(bestMap, bestSortMap);
                bestIndicator = (String) bestMap.get(0).get(ItemAttrConstants.NAME);
                bestIndicatorComplianceProjectCount = String.valueOf(
                        bestMap.get(0).get(AreaEnvContentCodes.qualifiedCount.name()));
            }
        }

        // 总项目数
        AttrValueVO projectCountVO = new AttrValueVO(
                AreaEnvHeaderCodes.projectsCount.name(), projectCount, AttrValueTypeEnum.number);
        results.add(projectCountVO);
        // 达标率最差指标
        AttrValueVO lowestComplianceRateIndicator = new AttrValueVO(
                AreaEnvHeaderCodes.lowestComplianceRateIndicator.name(), worstIndicator, AttrValueTypeEnum.str);
        results.add(lowestComplianceRateIndicator);
        // 最差指标不合格项目数
        AttrValueVO lowestIndicatorNoncomplianceProjectCount = new AttrValueVO(
                AreaEnvHeaderCodes.lowestIndicatorNoncomplianceProjectCount.name(),
                worstIndicatorNoncomplianceProjectCount, AttrValueTypeEnum.number);
        results.add(lowestIndicatorNoncomplianceProjectCount);
        // 达标率最好指标
        AttrValueVO highestComplianceRateIndicator = new AttrValueVO(
                AreaEnvHeaderCodes.highestComplianceRateIndicator.name(), bestIndicator, AttrValueTypeEnum.str);
        results.add(highestComplianceRateIndicator);
        // 最好指标合格项目数
        AttrValueVO highestIndicatorComplianceProjectCount = new AttrValueVO(
                AreaEnvHeaderCodes.highestIndicatorComplianceProjectCount.name(),
                bestIndicatorComplianceProjectCount, AttrValueTypeEnum.number);
        results.add(highestIndicatorComplianceProjectCount);

        return results;
    }

    @Override
    public List<AttrValueVO> getSummary(
            List<GroupVO> groupVOList, List<ItemVO> itemVOList, List<String> projectIdList) {
        return null;
    }


    @Override
    public ItemVO getIndicatorItem(Date reportDate, List<String> projectIds, String functionId) {
        ItemVO itemVO = new ItemVO();
        List<AttrValueVO> attrs = initItem(itemVO, functionId);

        Set<String> statisticCodeSet = Sets.newSet(
                ProjectEnvContentCodes.complianceRateIsQualified.name(),
                ProjectEnvContentCodes.complianceRate.name()
        );

        // 总项目数
        Integer projectsCountValue = projectIds.size();
        AttrValueVO projectsCount = new AttrValueVO(
                AreaEnvContentCodes.projectsCount.name(),
                String.valueOf(projectsCountValue), AttrValueTypeEnum.number);
        attrs.add(projectsCount);

        // 不合格项目数
        Integer unqualifiedProjectCount = null;
        // 合格项目数
        Integer qualifiedProjectCount = null;
        // 各项目达标率
        List<GraphItemVO> projectsComplianceRate = new ArrayList<>();
        for (String projectId : projectIds) {
            SimpleProjectVO simpleProjectInfo = saasWebService.getSimpleProjectInfo(projectId);
            if (simpleProjectInfo == null) {
                continue;
            }
            ItemVO projectItemValue = projectEnvBuilder.getProjectItemValue(
                    functionId, projectId, reportDate);
            Map<String, AttrValueVO> attrValueMap = attrValueList2Map(statisticCodeSet, projectItemValue.getAttrs());

            // 统计各项目达标率
            GraphItemVO graphItemVO = new GraphItemVO(simpleProjectInfo.getProjectLocalName(),
                    DataUtils.parseDouble(attrValueMap.get(ProjectEnvContentCodes.complianceRate.name()).getValue()),
                    AttrValueTypeEnum.number_2, DataConstants.PERCENT);
            projectsComplianceRate.add(graphItemVO);
            // 统计不合格项目数与合格项目数
            String isQualifiedDesc = attrValueMap.get(ProjectEnvContentCodes.complianceRateIsQualified.name()).getValue();
            if (QualifiedEnum.NOT_QUALIFIED.getDesc().equals(isQualifiedDesc)) {
                unqualifiedProjectCount = unqualifiedProjectCount == null ? 1 : unqualifiedProjectCount + 1;
            } else if (QualifiedEnum.QUALIFIED.getDesc().equals(isQualifiedDesc)) {
                qualifiedProjectCount = qualifiedProjectCount == null ? 1 : qualifiedProjectCount + 1;
            }
        }
        // 合格标准
        AttrValueVO qualifiedStandard = envFactory.getQualifiedStandard(
                functionService.queryFunctionDetail(functionId));
        attrs.add(qualifiedStandard);
        // 不合格项目数
        String unqualifiedProjectCountStr = unqualifiedProjectCount == null ?
                "0" : String.valueOf(unqualifiedProjectCount);
        AttrValueVO unqualifiedCount = new AttrValueVO(
                AreaEnvContentCodes.unqualifiedCount.name(),
                unqualifiedProjectCountStr, AttrValueTypeEnum.number);
        attrs.add(unqualifiedCount);
        // 合格项目数
        String qualifiedProjectCountStr = qualifiedProjectCount == null ?
                "0" : String.valueOf(qualifiedProjectCount);
        AttrValueVO qualifiedCount = new AttrValueVO(
                AreaEnvContentCodes.qualifiedCount.name(),
                qualifiedProjectCountStr, AttrValueTypeEnum.number);
        attrs.add(qualifiedCount);
        // 各项目达标率
        Map<String, Integer> sortMap = new HashMap<>();
        sortMap.put("value", -1);
        DataUtils.sort(projectsComplianceRate, sortMap);
        Gson gson = new Gson();
        GraphVO graphVO = new GraphVO(StandardNames.STANDARD_VALUE,
                qualifiedStandard.getValue(), AttrValueTypeEnum.percent_2, projectsComplianceRate);
        AttrValueVO projectComplianceRates = new AttrValueVO(
                AreaEnvContentCodes.projectComplianceRates.name(),
                gson.toJson(graphVO), AttrValueTypeEnum.graph);
        attrs.add(projectComplianceRates);

        return itemVO;
    }
}
