package com.persagy.apm.report.paragraphs.utils;

import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.paragraphs.constants.ItemAttrConstants;
import com.persagy.apm.report.paragraphs.service.impl.ParagraphBuilder;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * 段落工具类
 *
 * @author lixing
 * @version V1.0 2021/8/4 8:31 下午
 **/
public class ParagraphUtils {
    /**
     * 将条目转换为map
     *
     * @param itemVO  条目对象
     * @param codeSet 需要转换的属性集
     * @return 根据条目转换成的map对象
     * @author lixing
     * @version V1.0 2021/8/4 8:38 下午
     */
    public static Map<String, String> item2Map(ItemVO itemVO, Set<String> codeSet) {
        Map<String, String> itemMap = new HashMap<>();
        itemMap.put(ItemAttrConstants.ID, itemVO.getId());
        itemMap.put(ItemAttrConstants.NAME, itemVO.getName());
        Map<String, AttrValueVO> attrMap = ParagraphBuilder.attrValueList2Map(codeSet, itemVO.getAttrs());
        for (String key : attrMap.keySet()) {
            itemMap.put(key, attrMap.get(key).getValue());
        }
        return itemMap;
    }

    /**
     * map中value值数据类型转换
     *
     * @param map 源数据
     * @param memberName 需要修改value类型的key
     * @param memberType 需要修改成的类型
     * @return 修改后的map
     * @author lixing
     * @version V1.0 2021/9/7 2:26 下午
     */
    public static Map<String, Object> convertMapMemberType(
            Map<String, String> map, String memberName, Class<?> memberType) {
        if (CollectionUtils.isEmpty(map) || memberName == null || memberType == null) {
            return null;
        }
        Map<String, Object> anotherMap = new HashMap<>();

        try {
            for (String key : map.keySet()) {
                if (memberName.equals(key)) {
                    String s = map.get(key);
                    if (memberType.equals(Integer.class)) {
                        anotherMap.put(key, Integer.valueOf(s));
                    } else if (memberType.equals(Double.class)) {
                        anotherMap.put(key, Double.valueOf(s));
                    } else {
                        anotherMap.put(key, map.get(key));
                    }

                } else {
                    anotherMap.put(key, map.get(key));
                }
            }
        } catch (Exception e) {
            throw new RuntimeException("类型转换发生异常", e);
        }

        return anotherMap;
    }
}
