package com.persagy.apm.report.userconfig.paragraph.controller;

import com.persagy.apm.report.userconfig.paragraph.service.IParagraphService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.persagy.apm.report.userconfig.paragraph.model.*;
import com.persagy.apm.report.userconfig.paragraph.model.vo.*;
import com.persagy.apm.report.userconfig.paragraph.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 段落控制层
 *
 * @author lixing
 * @version V1.0 2021-07-15 17:46:11
 */
@Api(tags = "段落")
@ApiIgnore
@Validated
@RestController
@RequestMapping("paragraphs")
public class ParagraphController {
    @Autowired
    private IParagraphService paragraphService;

    @ApiOperation(value = "创建段落")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createParagraph(
            @Valid @RequestBody AddParagraphDTO addParagraphDTO) {
        String id = paragraphService.createParagraph(addParagraphDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "段落详情")
    @PostMapping("/get")
    public CommonResult<ParagraphItemVO> queryParagraphDetail(
            @Valid @RequestBody QueryParagraphDetailDTO queryParagraphDetailDTO) {
        Paragraph paragraph = paragraphService.queryParagraphDetail(queryParagraphDetailDTO.getId());
        ParagraphItemVO paragraphItemVO = ConvertParagraphTool.INSTANCE.convert2ResponseItemDTO(paragraph);
        return ResultHelper.single(paragraphItemVO);
    }


    @ApiOperation(value = "更新段落")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateParagraph(
            @Valid @RequestBody UpdateParagraphDTO updateParagraphDTO) {
        paragraphService.updateParagraph(updateParagraphDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateParagraphDTO.getId()));
    }


    @ApiOperation(value = "删除段落")
    @PostMapping("/delete")
    public CommonResult deleteParagraph(
            @RequestBody @Valid DeleteParagraphDTO deleteParagraphDTO) {
        paragraphService.deleteParagraph(deleteParagraphDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "段落列表")
    @PostMapping("/query")
    public CommonResult<PageList<ParagraphListItemVO>> queryParagraph(
            @Valid @RequestBody QueryParagraphDTO queryParagraphDTO) {
        List<Paragraph> paragraphList = paragraphService.queryParagraphList(queryParagraphDTO);
        List<ParagraphListItemVO> resultList = ConvertParagraphTool.INSTANCE.convert2List(paragraphList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "段落列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ParagraphListItemVO>> queryParagraph(
    //        @Valid @RequestBody PageQueryParagraphDTO pageQueryParagraphDTO) {
    //    IPage<Paragraph> pagedList = paragraphService.pageQueryParagraph(pageQueryParagraphDTO);
    //    IPage<ParagraphListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertParagraphTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，Paragraph实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

