package com.persagy.apm.report.userconfig.paragraph.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.report.userconfig.paragraph.model.vo.*;
import com.persagy.apm.report.userconfig.paragraph.model.dto.*;

import java.util.List;

/**
 * 段落(Paragraph) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-07-15 17:46:14 2021-07-15 17:46:14
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertParagraphTool {

    ConvertParagraphTool INSTANCE = Mappers.getMapper(ConvertParagraphTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param paragraph do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-07-15 17:46:14
     */
    ParagraphItemVO convert2ResponseItemDTO(Paragraph paragraph);

    /**
     * do转换为ResponseListItemVO
     *
     * @param paragraph do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-07-15 17:46:14
     */
    ParagraphListItemVO convert2ResponseListItemDTO(Paragraph paragraph);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param paragraphList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-07-15 17:46:14
     */
    List<ParagraphListItemVO> convert2List(List<Paragraph> paragraphList);

    /**
     * addDTO转换为do
     *
     * @param addParagraphDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-07-15 17:46:14
     */
    Paragraph convertAddDto2Entity(AddParagraphDTO addParagraphDTO);

    /**
     * updateDTO转换为实体
     *
     * @param paragraph          更新前的do对象
     * @param updateParagraphDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-07-15 17:46:14
     */
    Paragraph convertUpdateDto2Entity(@MappingTarget Paragraph paragraph, UpdateParagraphDTO updateParagraphDTO);
}

