package com.persagy.apm.report.userconfig.paragraphmodel.controller;

import com.persagy.apm.report.userconfig.paragraphmodel.service.IParagraphModelService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.persagy.apm.report.userconfig.paragraphmodel.model.*;
import com.persagy.apm.report.userconfig.paragraphmodel.model.vo.*;
import com.persagy.apm.report.userconfig.paragraphmodel.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 段落模板控制层
 *
 * @author lixing
 * @version V1.0 2021-07-20 10:26:09
 */
@Api(tags = "段落模板")
@ApiIgnore
@Validated
@RestController
@RequestMapping("paragraphModels")
public class ParagraphModelController {
    @Autowired
    private IParagraphModelService paragraphModelService;

    @ApiOperation(value = "创建段落模板")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createParagraphModel(
            @Valid @RequestBody AddParagraphModelDTO addParagraphModelDTO) {
        String id = paragraphModelService.createParagraphModel(addParagraphModelDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "段落模板详情")
    @PostMapping("/get")
    public CommonResult<ParagraphModelItemVO> queryParagraphModelDetail(
            @Valid @RequestBody QueryParagraphModelDetailDTO queryParagraphModelDetailDTO) {
        ParagraphModel paragraphModel = paragraphModelService.queryParagraphModelDetail(queryParagraphModelDetailDTO.getId());
        ParagraphModelItemVO paragraphModelItemVO = ConvertParagraphModelTool.INSTANCE.convert2ResponseItemDTO(paragraphModel);
        return ResultHelper.single(paragraphModelItemVO);
    }


    @ApiOperation(value = "更新段落模板")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateParagraphModel(
            @Valid @RequestBody UpdateParagraphModelDTO updateParagraphModelDTO) {
        paragraphModelService.updateParagraphModel(updateParagraphModelDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateParagraphModelDTO.getId()));
    }


    @ApiOperation(value = "删除段落模板")
    @PostMapping("/delete")
    public CommonResult deleteParagraphModel(
            @RequestBody @Valid DeleteParagraphModelDTO deleteParagraphModelDTO) {
        paragraphModelService.deleteParagraphModel(deleteParagraphModelDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "段落模板列表")
    @PostMapping("/query")
    public CommonResult<PageList<ParagraphModelListItemVO>> queryParagraphModel(
            @Valid @RequestBody QueryParagraphModelDTO queryParagraphModelDTO) {
        List<ParagraphModel> paragraphModelList = paragraphModelService.queryParagraphModelList(queryParagraphModelDTO);
        List<ParagraphModelListItemVO> resultList = ConvertParagraphModelTool.INSTANCE.convert2List(paragraphModelList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "段落模板列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ParagraphModelListItemVO>> queryParagraphModel(
    //        @Valid @RequestBody PageQueryParagraphModelDTO pageQueryParagraphModelDTO) {
    //    IPage<ParagraphModel> pagedList = paragraphModelService.pageQueryParagraphModel(pageQueryParagraphModelDTO);
    //    IPage<ParagraphModelListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertParagraphModelTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，ParagraphModel实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

