package com.persagy.apm.report.userconfig.pargraphdefine.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.report.userconfig.paragraphmodel.model.ParagraphModel;
import com.persagy.apm.report.userconfig.paragraphmodel.service.IParagraphModelService;
import com.persagy.apm.report.userconfig.pargraphdefine.dao.ParagraphDefineMapper;
import com.persagy.apm.report.userconfig.pargraphdefine.service.IParagraphDefineService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.report.userconfig.pargraphdefine.model.*;
import com.persagy.apm.report.userconfig.pargraphdefine.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 段落定义(ParagraphDefine) service层
 *
 * @author lixing
 * @version V1.0 2021-07-20 10:23:08
 */
@Service
public class ParagraphDefineServiceImpl extends ServiceImpl<ParagraphDefineMapper, ParagraphDefine>
        implements IParagraphDefineService {
    @Autowired
    private IParagraphModelService paragraphModelService;

    /**
     * 创建段落定义
     *
     * @return 段落定义主键
     * @author lixing
     * @version V1.0 2021-07-20 10:23:08
     */
    @Override
    public String createParagraphDefine(AddParagraphDefineDTO addParagraphDefineDTO) {
        ParagraphDefine paragraphDefine = ConvertParagraphDefineTool.INSTANCE.convertAddDto2Entity(addParagraphDefineDTO);
        // 设置默认值
        setDefaultValue(paragraphDefine);
        save(paragraphDefine);
        return paragraphDefine.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param paragraphDefine 段落定义实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ParagraphDefine paragraphDefine) {
        paragraphDefine.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 段落定义详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-07-20 10:23:08
     */
    @Override
    public ParagraphDefine queryParagraphDefineDetail(String id) {
        ParagraphDefine paragraphDefine = getById(id);
        if (paragraphDefine == null) {
            throw new IllegalArgumentException("查看ParagraphDefine详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return paragraphDefine;
    }

    /**
     * 更新段落定义
     *
     * @author lixing
     * @version V1.0 2021-07-20 10:23:08
     */
    @Override
    public void updateParagraphDefine(UpdateParagraphDefineDTO updateParagraphDefineDTO) {
        ParagraphDefine paragraphDefine = getById(updateParagraphDefineDTO.getId());
        paragraphDefine = ConvertParagraphDefineTool.INSTANCE.convertUpdateDto2Entity(paragraphDefine, updateParagraphDefineDTO);
        paragraphDefine.setModifier(AppContext.getContext().getAccountId());
        updateById(paragraphDefine);
    }

    /**
     * 校验段落定义是否可删除
     *
     * @param id 段落定义主键
     * @return 段落定义do类
     * @author lixing
     * @version V1.0 2021-07-20 10:23:08
     */
    public ParagraphDefine checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ParagraphDefine时发生异常，主键为空");
        }

        ParagraphDefine paragraphDefine = getById(id);

        if (paragraphDefine == null) {
            throw new IllegalArgumentException("删除ParagraphDefine时发生异常，找不到要删除的数据，id:" + id);
        }

        return paragraphDefine;
    }

    /**
     * 删除段落定义
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-07-20 10:23:08
     */
    @Override
    public void deleteParagraphDefine(String id) {
        // 校验是否可删除
        ParagraphDefine paragraphDefine = checkDeletable(id);

        paragraphDefine.setValid(ValidEnum.FALSE.getType());
        updateById(paragraphDefine);
    }

    /**
     * 查询段落定义
     *
     * @return List<ParagraphDefine>
     * @author lixing
     * @version V1.0 2021-07-20 10:23:08
     */
    @Override
    public List<ParagraphDefine> queryParagraphDefineList(QueryParagraphDefineDTO queryParagraphDefineDTO) {
        QueryWrapper<ParagraphDefine> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ParagraphDefine.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ParagraphDefine.PROP_CREATIONTIME);

        if (queryParagraphDefineDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryParagraphDefineDTO.getModelId())) {
                queryWrapper.like(ParagraphDefine.PROP_MODEL_ID, queryParagraphDefineDTO.getModelId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryParagraphDefineDTO.getGroupIds())) {
                queryWrapper.like(ParagraphDefine.PROP_GROUP_IDS, queryParagraphDefineDTO.getGroupIds());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryParagraphDefineDTO.getItemIds())) {
                queryWrapper.like(ParagraphDefine.PROP_ITEM_IDS, queryParagraphDefineDTO.getItemIds());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询段落定义
     *
     * @return IPage<ParagraphDefine>
     * @author lixing
     * @version V1.0 2021-07-20 10:23:08
     */
    @Override
    public IPage<ParagraphDefine> pageQueryParagraphDefine(PageQueryParagraphDefineDTO pageQueryParagraphDefineDTO) {
        QueryWrapper<ParagraphDefine> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ParagraphDefine.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ParagraphDefine> pageParam = new Page<>(pageQueryParagraphDefineDTO.getPage(), pageQueryParagraphDefineDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryParagraphDefineDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ParagraphDefine.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryParagraphDefineDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryParagraphDefineDTO.getModelId())) {
            queryWrapper.like(ParagraphDefine.PROP_MODEL_ID, pageQueryParagraphDefineDTO.getModelId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryParagraphDefineDTO.getGroupIds())) {
            queryWrapper.like(ParagraphDefine.PROP_GROUP_IDS, pageQueryParagraphDefineDTO.getGroupIds());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryParagraphDefineDTO.getItemIds())) {
            queryWrapper.like(ParagraphDefine.PROP_ITEM_IDS, pageQueryParagraphDefineDTO.getItemIds());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public String getModelCode(String defineId) {
        ParagraphDefine paragraphDefine = queryParagraphDefineDetail(defineId);
        return getModelCode(paragraphDefine);
    }

    @Override
    public String getModelCode(ParagraphDefine paragraphDefine) {
        // 获取段落模板信息
        ParagraphModel paragraphModel = paragraphModelService.queryParagraphModelDetail(paragraphDefine.getModelId());
        return paragraphModel.getCode();
    }
}
